#ifndef __COMMON_DEF_H__
#define __COMMON_DEF_H__

#include <stdint.h>
#include <stddef.h>
#include "plc_abi.h"
#include "iec/iec_std_lib.h"
#include "user_def.h"

#define MEM_BYTE_SIZE           __ALIGN64(5120)
#define INPUT_BYTE_SIZE         __ALIGN64(96)
#define OUTPUT_BYTE_SIZE        __ALIGN64(48)

// The following 3 rows will be replaced by STCompiler.exe.
IEC_MEM_BLOCK_DEF(MemM, MEM_BYTE_SIZE, 
    // X
    struct
    {
        uint8_t rev1 [4096];
        struct
        {
            uint8_t B1 : 1;    // %MX4096.0, size:{1 byte, 1 X}
            uint8_t B2 : 1;    // %MX4096.1, size:{1 byte, 1 X}
            uint8_t B3 : 1;    // %MX4096.2, size:{1 byte, 1 X}
            uint8_t B4 : 1;    // %MX4096.3, size:{1 byte, 1 X}
            uint8_t B5 : 1;    // %MX4096.4, size:{1 byte, 1 X}
            uint8_t B6 : 1;    // %MX4096.5, size:{1 byte, 1 X}
            uint8_t B7 : 1;    // %MX4096.6, size:{1 byte, 1 X}
            uint8_t B8 : 1;    // %MX4096.7, size:{1 byte, 1 X}
        };
        struct
        {
            uint8_t B9 : 1;    // %MX4097.0, size:{1 byte, 1 X}
            uint8_t B10 : 1;    // %MX4097.1, size:{1 byte, 1 X}
        };
    };

    // W
    struct
    {
        uint16_t rev2 [2052];
        INT A1;    // %MW2052, size:{16 byte, 1 W}
        INT A2;    // %MW2053, size:{16 byte, 1 W}
        INT A3;    // %MW2054, size:{16 byte, 1 W}
        INT A4;    // %MW2055, size:{16 byte, 1 W}
        INT A5;    // %MW2056, size:{16 byte, 1 W}
        INT A6;    // %MW2057, size:{16 byte, 1 W}
        INT A7;    // %MW2058, size:{16 byte, 1 W}
        INT A8;    // %MW2059, size:{16 byte, 1 W}
        INT A9;    // %MW2060, size:{16 byte, 1 W}
        INT A10;    // %MW2061, size:{16 byte, 1 W}
    };
);
IEC_MEM_BLOCK_DEF(MemI, INPUT_BYTE_SIZE, struct { };);
IEC_MEM_BLOCK_DEF(MemQ, OUTPUT_BYTE_SIZE, struct { };);

// The following 10 rows will be updated by FSU backend after STCompiler.exe finished.
#define __PORT_1_INDEX 255
#define __PORT_2_INDEX 255
#define __PORT_3_INDEX 255
#define __PORT_4_INDEX 255
#define __PORT_10_INDEX 255
#define __PORT_1_PERI_NAME NULL
#define __PORT_2_PERI_NAME NULL
#define __PORT_3_PERI_NAME NULL
#define __PORT_4_PERI_NAME NULL
#define __PORT_10_PERI_NAME NULL

#define MC_GROUP_MAX_AXIS_NUM 4

extern MemM __mem;
extern MemI __mem_input;
extern MemQ __mem_output;

extern int plc_create_task(const char* task_name, uint32_t priority, uint64_t interval_us, PlcTaskFuncInit task_init, PlcTaskFuncRun task_run, PlcTaskFuncTrigger task_trigger);
extern void* plc_find_sys_api(const char* api_id, void** api_context);
extern void* plc_find_sys_var(const char* var_id, uint8_t* bit_mask);
extern void plc_update_start(void);
extern void plc_update_end(void);

#endif
